package com.chelvc.framework.wechat.config;

import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.wechat.DefaultWechatAppletHandler;
import com.chelvc.framework.wechat.DefaultWechatPaymentHandler;
import com.chelvc.framework.wechat.DefaultWechatPublicHandler;
import com.chelvc.framework.wechat.WechatAppletHandler;
import com.chelvc.framework.wechat.WechatPublicHandler;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.BeansException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.config.BeanPostProcessor;
import org.springframework.beans.factory.support.RootBeanDefinition;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.support.GenericApplicationContext;

/**
 * 微信配置类
 *
 * @author Woody
 * @date 2024/1/30
 */
@Configuration
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class WechatConfigurer implements BeanPostProcessor {
    private final WechatProperties properties;
    private final GenericApplicationContext applicationContext;
    private volatile boolean initialized;

    /**
     * 初始化微信操作处理器
     */
    private void initializeWechatHandler() {
        // 注册小程序操作Bean
        if (ObjectUtils.notEmpty(this.properties.getApplets())) {
            this.properties.getApplets().forEach(config -> {
                WechatAppletHandler handler = new DefaultWechatAppletHandler(config);
                String name = StringUtils.ifEmpty(config.getName(), config.getAppid());
                RootBeanDefinition definition = new RootBeanDefinition(WechatAppletHandler.class, () -> handler);
                this.applicationContext.registerBeanDefinition(name, definition);
            });
        }

        // 注册公众号操作Bean
        if (ObjectUtils.notEmpty(this.properties.getPublics())) {
            this.properties.getPublics().forEach(config -> {
                WechatPublicHandler handler = new DefaultWechatPublicHandler(config);
                String name = StringUtils.ifEmpty(config.getName(), config.getAppid());
                RootBeanDefinition definition = new RootBeanDefinition(WechatPublicHandler.class, () -> handler);
                this.applicationContext.registerBeanDefinition(name, definition);
            });
        }

        // 注册微信支付操作Bean
        if (ObjectUtils.notEmpty(this.properties.getPayments())) {
            this.applicationContext.registerBean(DefaultWechatPaymentHandler.class, this.properties.getPayments());
        }
    }

    @Override
    public synchronized Object postProcessBeforeInitialization(Object bean, String beanName) throws BeansException {
        if (!this.initialized) {
            this.initialized = true;
            this.initializeWechatHandler();
        }
        return bean;
    }
}
